IF OBJECT_ID('TR__ServerRoles__CircularDependencyCheck') IS NOT NULL
DROP TRIGGER [TR__ServerRoles__CircularDependencyCheck]
GO

CREATE TRIGGER [TR__ServerRoles__CircularDependencyCheck]
ON [ServerRoleDependencies] AFTER UPDATE, INSERT AS
BEGIN

	SET NOCOUNT ON

	DECLARE @Deps TABLE(
		[Base_Id] INT NOT NULL, 

		[Resolution_Depth] INT NOT NULL,
		[Id] INT NOT NULL, 
		[DependsOn_Id] INT NULL
	)

	DECLARE @depth INT, @row_count INT
	    SET @depth = 1
	    SET @row_count = 1 
	 
	INSERT INTO @Deps 
	  ([Base_Id], [Resolution_Depth], [Id], [DependsOn_Id])
	SELECT 
	   [ServerRole_Id], 1, [ServerRole_Id], [DependsOn_ServerRole_Id]
	  FROM [inserted] 
     

	WHILE @row_count > 0
	BEGIN 
		SET @depth = @depth + 1

		INSERT INTO @Deps 
		  ([Base_Id], [Resolution_Depth], [Id], [DependsOn_Id])
		SELECT
		  [Base_Id], @depth, D.[DependsOn_Id], SRD.[DependsOn_ServerRole_Id]
		FROM @Deps D
	         INNER JOIN [ServerRoleDependencies] SRD
		             ON SRD.[ServerRole_Id] = D.[DependsOn_Id]
		WHERE D.[Resolution_Depth] = @depth - 1

		SET @row_count = @@ROWCOUNT
	
		IF EXISTS( SELECT * 
		             FROM [inserted] I
		                  INNER JOIN @Deps D
		                          ON D.[Resolution_Depth] > 1
		                         AND I.[ServerRole_Id] = D.[Base_Id]
		                         AND I.[ServerRole_Id] = D.[DependsOn_Id] )

		BEGIN 
				RAISERROR('The specified dependent ID will cause a cycle at depth %i, which is not supported.',16,2, @depth)
				ROLLBACK
				RETURN 
		END 

		-- sanity check
		IF @depth > 1000 BEGIN
				RAISERROR('Depth exceeded 1000.',16,1) 
				ROLLBACK
				RETURN 
		END 
	END 


END
GO