EXEC [dbo].[__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_CreateOrUpdateRole',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Role_Id',
    /* Description_Text        */ 'Creates a new role defined by the specified tasks'
GO

IF OBJECT_ID('[dbo].[Security_CreateOrUpdateRole]') IS NOT NULL
	DROP PROCEDURE [dbo].[Security_CreateOrUpdateRole]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Security_CreateOrUpdateRole]
(
	@Role_Id INT = NULL OUT,
	@Role_Name NVARCHAR(50),
        @Role_Description NVARCHAR(MAX),
	@Tasks_Csv VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

        DECLARE @EnvironmentScopable_Indicator YNINDICATOR
            SET @EnvironmentScopable_Indicator = CASE
                WHEN EXISTS (SELECT * FROM [Tasks] 
                              WHERE [EnvironmentScopable_Indicator] = 'N'
                               AND [Task_Id] IN (SELECT [Value_Text] FROM [CsvToTable](@Tasks_Csv, ',')))
                     THEN 'N' ELSE 'Y' END

	IF @Role_Id IS NULL
	BEGIN
		/* Add new role */
		INSERT INTO [Roles] ([Role_Name], [Role_Description], [EnvironmentScopable_Indicator])
		VALUES (@Role_Name, @Role_Description, @EnvironmentScopable_Indicator)
		SET @Role_Id = SCOPE_IDENTITY()

		-- EXEC [Events_RaiseEvent] 'SECNWR', NULL, @Role_Id

	END
	ELSE
	BEGIN
		/* Update existing role */
		UPDATE [Roles]
		   SET [Role_Name] = @Role_Name,
                       [Role_Description] = @Role_Description,
                       [EnvironmentScopable_Indicator] = @EnvironmentScopable_Indicator
		 WHERE [Role_Id] = @Role_Id

		/* Remove existing tasks for the role */
		DELETE [RoleTasks]
		 WHERE [Role_Id] = @Role_Id

		-- EXEC [Events_RaiseEvent] 'SECMDR', NULL, @Role_Id
	END

	/* Add the appropriate role tasks */
	INSERT INTO [RoleTasks] ([Role_Id] ,[Task_Id])
	SELECT @Role_Id,[Value_Text]
	  FROM [dbo].[CsvToTable](@Tasks_Csv, ',')

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [dbo].[Security_CreateOrUpdateRole] TO [OtterUser_Role]
GO
